@@ -510,6 +510,8 @@ namespace CEC
         return "Vizio";
       case CEC_VENDOR_BROADCOM:
         return "Broadcom";
+      case CEC_VENDOR_LOEWE:
+        return "Loewe";
       default:
         return "Unknown";
       }
@@ -547,6 +549,14 @@ namespace CEC
         return "1.8.0";
       case CEC_CLIENT_VERSION_1_8_1:
         return "1.8.1";
+      case CEC_CLIENT_VERSION_1_8_2:
+        return "1.8.2";
+      case CEC_CLIENT_VERSION_1_9_0:
+        return "1.9.0";
+      case CEC_CLIENT_VERSION_1_99_0:
+        return "2.0.0-pre";
+      case CEC_CLIENT_VERSION_2_0_0:
+        return "2.0.0";
       default:
         return "Unknown";
       }
@@ -584,6 +594,14 @@ namespace CEC
         return "1.8.0";
       case CEC_SERVER_VERSION_1_8_1:
         return "1.8.1";
+      case CEC_SERVER_VERSION_1_8_2:
+        return "1.8.2";
+      case CEC_SERVER_VERSION_1_9_0:
+        return "1.9.0";
+      case CEC_SERVER_VERSION_1_99_0:
+        return "2.0.0-pre";
+      case CEC_CLIENT_VERSION_2_0_0:
+        return "2.0.0";
       default:
         return "Unknown";
       }
@@ -776,5 +794,22 @@ namespace CEC
         return "unknown";
       }
     }
+
+    static const char *ToString(const cec_adapter_type type)
+    {
+      switch (type)
+      {
+      case ADAPTERTYPE_P8_EXTERNAL:
+        return "Pulse-Eight USB-CEC Adapter";
+      case ADAPTERTYPE_P8_DAUGHTERBOARD:
+        return "Pulse-Eight USB-CEC Daughterboard";
+      case ADAPTERTYPE_RPI:
+        return "Raspberry Pi";
+      case ADAPTERTYPE_TDA995x:
+        return "TDA995x";
+      default:
+        return "unknown";
+      }
+    }
   };
 }
